/*****************************************************************************
 * VLCMedialistDataSource.m: VLC.framework VLCMedialistDataSource implementation
 *****************************************************************************
 * Copyright (C) 2007 Pierre d'Herbemont
 * Copyright (C) 2007 the VideoLAN team
 * $Id: VLCPlaylistDataSource.m 21564M 2007-10-01 21:38:59Z (local) $
 *
 * Authors: Pierre d'Herbemont <pdherbemont # videolan.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
 *****************************************************************************/

#import <VLCMedialistDataSource.h>
#import "VLCEventManager.h"

@implementation VLCMedialistDataSource

- (id)init
{
    if (self = [super init])
    {
        playlist = nil;
        videoView = nil;
    }
    return self;
}

- (id)initWithPlaylist:(VLCMedialist *)aPlaylist
{
    if (self = [super init])
    {

        playlist = [aPlaylist retain];

        [[NSNotificationCenter defaultCenter] addObserver: self selector: @selector(playlistDidChange:) name:VLCMedialistItemAdded  object:nil];
        [[NSNotificationCenter defaultCenter] addObserver: self selector: @selector(itemDidAddSubitem:) name:VLCMediaSubItemAdded  object:nil];
        [[NSNotificationCenter defaultCenter] addObserver: self selector: @selector(itemDidChange:) name:VLCMedialistItemChanged   object:nil];
        videoView = nil;
        outlineView = nil;

    }
    return self;
}

- (id)initWithPlaylist:(VLCMedialist *)aPlaylist videoView:(VLCVideoView *)aVideoView;
{
    if (self = [super init])
    {
        playlist = [aPlaylist retain];

        [[NSNotificationCenter defaultCenter] addObserver: self selector: @selector(playlistDidChange:) name:VLCMedialistItemAdded object:nil];
        [[NSNotificationCenter defaultCenter] addObserver: self selector: @selector(itemDidAddSubitem:) name:VLCMediaSubItemAdded object:nil];
        [[NSNotificationCenter defaultCenter] addObserver: self selector: @selector(itemDidChange:) name:VLCMedialistItemChanged   object:nil];

        videoView = [aVideoView retain];
        /* Will be automatically set if an outline view ask us data,
         * be careful not to connect two outlineView to this object or this goes wrong. */
        outlineView = nil;
    }
    return self;
}

- (void)dealloc
{
    [[NSNotificationCenter defaultCenter] removeObserver: self];

    if (playlist)
        [playlist release];
    if (videoView)
        [videoView release];

    [super dealloc];
}

- (VLCMedialist *)playlist
{
    return playlist;
}

- (VLCVideoView *)videoView
{
    return videoView;
}
@end



@implementation VLCMedialistDataSource (OutlineViewDataSource)
- (BOOL)  outlineView: (NSOutlineView *)ov isItemExpandable: (id)item 
{ 
	return NO; 
}

- (int)   outlineView: (NSOutlineView *)ov numberOfChildrenOfItem:(id)item 
{ 
	if (item == nil)
		return [playlist countMedia];
	return 0; 
}

- (id)    outlineView: (NSOutlineView *)ov child:(int)index ofItem:(id)item 
{
	if (item == nil)
		return [playlist mediaAtIndex:index];
	return nil; 
}

- (id)    outlineView: (NSOutlineView *)ov objectValueForTableColumn:(NSTableColumn*)col byItem:(id)item 
{ 
	if (item != nil)
	{
		return [(VLCMedia *)item url];
	}
	return nil; 
}

@end

@implementation VLCMedialistDataSource (TableViewDropping)
/* Dummy implementation cause we need them */
- (int)numberOfRowsInTableView:(NSTableView *)aTableView 
{ 
	return [playlist countMedia];
}

- (id)tableView:(NSTableView *)aTableView objectValueForTableColumn:(NSTableColumn *)aTableColumn row:(int)rowIndex 
{
	return [[playlist mediaAtIndex:rowIndex] url];
}

- (NSDragOperation)tableView:(NSTableView*)tv validateDrop:(id <NSDraggingInfo>)info proposedRow:(int)row proposedDropOperation:(NSTableViewDropOperation)op
{
    return NSDragOperationEvery; /* This is for now */
}

- (BOOL)tableView:(NSTableView *)aTableView acceptDrop:(id <NSDraggingInfo>)info
            row:(int)row dropOperation:(NSTableViewDropOperation)operation
{
    int i;
    NSArray  * droppedItems = [[info draggingPasteboard] propertyListForType: NSFilenamesPboardType];
 
    for (i = 0; i < [droppedItems count]; i++)
    {
        NSString * filename = [droppedItems objectAtIndex:i];
        
        [[self playlist] insertMedia:[VLCMedia mediaWithURL:filename] atIndex:row+i];
    }
	return YES;

}
@end

@interface NSObject (UnknownBindingsObject)
/* OutlineViewDataSourceDropping and bindings hack */
- (id)observedObject;
@end

@implementation VLCMedialistDataSource (OutlineViewDataSourceDropping)
- (BOOL)outlineView:(NSOutlineView *)aOutlineView acceptDrop:(id <NSDraggingInfo>)info item:(id)item childIndex:(int)index
{
    NSArray  * droppedItems = [[info draggingPasteboard] propertyListForType: NSFilenamesPboardType];
    VLCMedialist * aPlaylist;
    int i;

    if (!item)
        item = [self playlist]; /* The root object is our playlist */
    else
        item = [item observedObject];

    if (![item  isMemberOfClass:[VLCMedialist class]])
        return NO;

    if (index < 0) /* No precise item given, put it as the first one */
        index = 0;

    aPlaylist = item;

    if (!droppedItems)
    {
        /* XXX: localization */
        NSRunCriticalAlertPanelRelativeToWindow(@"Error", @"Unable to drop the provided item.", @"OK", nil, nil, [outlineView window]);
        return NO;
    }

    for (i = 0; i < [droppedItems count]; i++)
    {
        NSString * filename = [droppedItems objectAtIndex:i];
        
        [aPlaylist insertMedia:[VLCMedia mediaWithURL:filename] atIndex:index+i];
    }
    return YES;
}

- (NSDragOperation)outlineView:(NSOutlineView *)aOutlineView validateDrop:(id <NSDraggingInfo>)info proposedItem:(id)item proposedChildIndex:(int)index
{
    return NSDragOperationEvery;
}
@end